<?php

namespace Vtbpay\Classes\Api;

use \Vtbpay\Classes\Common\Request,
    \Vtbpay\Classes\Exception\VtbPayException;

/**
 * Класс VtbApi предоставляет методы для взаимодействия с API VTB.
 */
class VtbApi extends Request
{
    private string $mode;
    private string $clientMerchant;
    private array $clientData;

    private static array $modeUrls = [
        'PROD' => [
            'TOKEN' => 'https://open.api.vtb.ru:443/passport/oauth2/token',
            'ORDER' => 'https://gw.api.vtb.ru/openapi/smb/efcp/e-commerce/v1/orders',
            'REFUND' => 'https://gw.api.vtb.ru/openapi/smb/efcp/e-commerce/v1/refunds'
        ],
        'TEST' => [
            'TOKEN' => 'https://auth.bankingapi.ru/auth/realms/kubernetes/protocol/openid-connect/token',
            'ORDER' => 'https://hackaton.bankingapi.ru/api/smb/efcp/e-commerce/api/v1/orders',
            'REFUND' => 'https://hackaton.bankingapi.ru/api/smb/efcp/e-commerce/api/v1/refunds'
        ]
    ];


    /**
     * Конструктор класса VtbApi.
     *
     * @param string $clientId     Идентификатор клиента.
     * @param string $clientSecret Секрет клиента.
     * @param bool   $test         Режим тестирования (по умолчанию - false).
     * @param string $merchant     Идентификатор продавца (по умолчанию - пустая строка).
     *
     * @throws VtbPayException Если не заполнены обязательные поля $clientId или $clientSecret.
     */
    public function __construct(
        string $clientId,
        string $clientSecret,
        bool $test = false,
        string $merchant = ''
    ) {
        if (empty($clientId) || empty($clientSecret)) {
            throw new VtbPayException('Обязательные поля не заполнены.');
        }

        $this->clientData = [
            'client_id' => trim($clientId),
            'client_secret' => trim($clientSecret),
            'grant_type' => 'client_credentials'
        ];

        $this->clientMerchant = $merchant;
        $this->mode = $test ? 'TEST' : 'PROD';
    }
    

    /**
     * Получает URL для заданного типа запроса.
     *
     * @param string $type Тип запроса ('TOKEN', 'ORDER' или 'REFUND').
     *
     * @return string URL для запроса.
     */
    private function getUrl(string $type): string
    {
        return self::$modeUrls[$this->mode][$type] ?? '';
    }


    /**
     * Генерирует массив заголовков для запроса.
     *
     * @param array  $extraHeaders    Дополнительные заголовки.
     * @param bool   $authorization   Флаг авторизации (по умолчанию - false).
     *
     * @return array Массив заголовков для запроса.
     */
    private function getHeaders(array $extraHeaders = [], bool $authorization = false): array
    {
        $headers = $extraHeaders;
        
        if ($authorization) {
            $headers = array_merge($this->getAuthorizationHeaders(), $headers);
        }
        
        if (!empty($this->clientMerchant)) {
            $headers[] = 'Merchant-Authorization: ' . $this->clientMerchant;
        }
        
        return $headers;
    }


    /**
     * Получает заголовки авторизации.
     *
     * @return array Массив заголовков для авторизации.
     *
     * @throws VtbPayException Если не удается получить токен доступа.
     */
    private function getAuthorizationHeaders(): array
    {
        $token = $this->getAccessToken();
        
        if (!isset($token['access_token']) || !isset($token['token_type'])) {
            $errorDescription = isset($token['error_description']) ? ' Error description: ' . $token['error_description'] : '';
            throw new VtbPayException('Ошибка при получении токена доступа.' . $errorDescription, [
                'token' => $token
            ]);
        }
        
        return [
            'X-IBM-Client-Id: ' . explode('@', $this->clientData['client_id'])[0],
            'Authorization: ' . $token['token_type'] . ' ' . $token['access_token'],
        ];
    }


    /**
     * Получает массив с данными токена доступа.
     *
     * @return array Массив с токеном доступа.
     */
    private function getAccessToken(): array
    {
        $response = $this->sendRequest(
            $this->getUrl('TOKEN'),
            http_build_query($this->clientData),
            $this->getHeaders([
                'Content-Type: application/x-www-form-urlencoded'
            ])
        );

        return $response;
    }


    /**
     * Получает сгенерированный данные о странице оплаты заказа.
     *
     * @param string $orderId     Идентификатор заказа.
     * @param string $email       Email покупателя.
     * @param int    $orderCreate Дата создания заказа.
     * @param float  $amount      Сумма заказа.
     * @param string $returnUrl   URL для возврата после оплаты.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если не удается получить ссылки для оплаты.
     */
    public function getOrderLink(
        string $orderId,
        string $email,
        int $orderCreate,
        float $amount,
        string $returnUrl
    ): array {
        $headers = $this->getHeaders(['Content-Type: application/json'], true);
        $timestamp = $orderCreate + 86400; // +1 day
        $expire = date('Y-m-d\TH:i:s.v\Z', $timestamp); // ISO 8601
        
        $response = $this->sendRequest(
            $this->getUrl('ORDER'), [
                'orderId' => $orderId,
                'orderName' => 'Заказ #' . $orderId,
                'customer' => ['email' => $email],
                'amount' => ['value' => $amount, 'code' => 'RUB'],
                'expire' => $expire,
                'returnUrl' => $returnUrl
            ],
            $headers
        );
        
        if (
            !isset($response['object']['payUrl']) ||
            empty($response['object']['payUrl'])
        ) {
            throw new VtbPayException('Ошибка при получении ссылки для оплаты.', [
                'response' => $response
            ]);
        }

        return $response;
    }


    /**
     * Получает информацию о заказе.
     *
     * @param string $orderId Идентификатор заказа.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если не удается получить статус платежа.
     */
    public function getOrderInfo(string $orderId): array
    {
        $headers = $this->getHeaders([], true);

        $response = $this->sendRequest(
            $this->getUrl('ORDER') . '/' . $orderId,
            [],
            $headers,
            'GET'
        );
        
        if (
            !isset($response['object']['status']['value']) ||
            empty($response['object']['status']['value'])
        ) {
            throw new VtbPayException('Информация о статусе платежа отсутствует.', [
                'response' => $response
            ]);
        }
        
        return $response;
    }


    /**
     * Устанавливает возврат средств.
     *
     * @param string $orderId Идентификатор заказа.
     *
     * @return array Результат запроса.
     * @throws VtbPayException Если нет информации об платеже
     */
    public function setRefunds(string $orderId): array
    {
        $headers = $this->getHeaders(['Content-Type: application/json'], true);
        $response = $this->getOrderInfo($orderId);
        $transactionPayment = $response['object']['transactions']['payments'][0]['object'] ?? '';

        if (empty($transactionPayment)) {
            throw new VtbPayException('Информация о платеже отсутствует.', [
                'response' => $response
            ]);
        }

        return $this->sendRequest(
            $this->getUrl('REFUND'),
            [
                'refundId' => $orderId . '-REFUND-' . time(),
                'paymentId' => $transactionPayment['paymentId'],
                'amount' => [
                    'value' => $transactionPayment['amount']['value'],
                    'code' => $transactionPayment['amount']['code']
                ]
            ],
            $headers
        );
    }
}
